/***********************************************************************/
/*                                                                     */
/*   svm_donlp2_c.c                                                    */
/*                                                                     */
/*   Interface to the DONLP2_c optimization package for SVM.           */
/*                                                                     */
/*   Copyright: Thorsten Joachims                                      */
/*   Date: 11.11.97                                                    */
/*                                                                     */
/***********************************************************************/

/* Common Block Declarations */

# define NX        100    /* these values MUST not be changed. the maximum */
# define NRESM     1800   /* value for -q option is 100! */

long verbosity;

struct {
    double x[NX], x0[NX], x1[NX], xmin[NX], resmin[NRESM], d__[NX], 
	    d0[NX], dd[NX], difx[NX], xnorm, x0norm, dnorm, d0norm, sig, 
	    sig0, sigmin, dscal, upsi, upsi0, upsi1, upsist, upsim, psi, psi0,
	    psi1, psist, psimin, phi, phi0, phi1, phimin, fx, fx0, fx1, fxst,
	    fmin, b2n, b2n0, dirder, cosphi;
} o8xdat_;

#define o8xdat_1 o8xdat_

struct {
    long val[NRESM+1], gconst[NRESM+1];
    long gunit[(NRESM+1)*3]	/* was [3][NRESM+1] */;
    long llow[NX], lup[NX];
} o8gri_;

#define o8gri_1 o8gri_

struct {
    long intakt, inx, std, te0, te1, te2, te3, singul, ident, eqres, 
	 silent, analyt, cold;
} o8stpa_;

#define o8stpa_1 o8stpa_

struct {
    long n, nh, ng, nr, nres;
} o8dim_;

#define o8dim_1 o8dim_

struct {
    double del0, del01, del, delmin, tau0, tau, smalld, smallw, rho, rho1,
	   eta, ny, epsx, c1d, scfmax, tauqp, taufac, taumax, updmy0;
    long iterma, ifill1;
} o8par_;

#define o8par_1 o8par_

struct {
    char name__[40];
} o8id_;

#define o8id_1 o8id_

struct {
    double epsdif;
} o8der_;

#define o8der_1 o8der_

struct {
    long prou, meu;
} o8io_;

#define o8io_1 o8io_

/* /////////////////////////////////////////////////////////////// */

int nx = NX;

/* /////////////////////////////////////////////////////////////// */

double *optimize_qp();
double svm_c;
double *opt_ce,*opt_ce0,*opt_g,*opt_g0,*opt_xinit;
long opt_n;

/* /////////////////////////////////////////////////////////////// */

double *optimize_qp()
/* start the optimizer and return the optimal values */
{
  donlp2_();
  return(o8xdat_1.x);
}

/* /////////////////////////////////////////////////////////////// */
int setup_()
{
/*   COLNO ALSO USED O8QPSO WITH DOUBLE LENGTH ! */
    o8par_1.epsx = 1e-7;

    return 0;
} 

int setup0_()
{
    int s_copy();
    long i,j;

    o8dim_1.n = opt_n;                 /* number of examples */
    for (i = 0; i < o8dim_1.n; i++) {  /* set starting values */
	o8xdat_1.x[i] = opt_xinit[i];
    }
    s_copy(o8id_1.name__, "SVMXQP", 40L, 6L); /* name of output file */
    o8dim_1.nh = 1;                    /* number of linear constraints */
    o8dim_1.ng = 2*opt_n;              /* number of bound constraints */
    o8stpa_1.silent = 1;               /* FALSE = be verbose */
    if(verbosity>=3) {
      o8stpa_1.silent = 0; 
    }
    o8stpa_1.analyt = 1;
    o8io_1.prou = 10;
    o8io_1.meu = 20;
    o8der_1.epsdif = 0.;
    o8par_1.del0 = 1.;
    o8par_1.tau0 = 10;

    o8gri_1.gunit[0] = -1;   /* for the  objective function */ 
    o8gri_1.gunit[1] = 0; 
    o8gri_1.gunit[2] = 0; 
    o8gri_1.gunit[3+0] = -1; /* for the linear equality constraint */ 
    o8gri_1.gunit[3+1] = 0; 
    o8gri_1.gunit[3+2] = 0; 
    for (j = 1; j <= opt_n; j++) { /* for the boundary constraints */ 
	o8gri_1.gunit[(j*6) + 0] = 1;
	o8gri_1.gunit[(j*6) + 1] = j; 
	o8gri_1.gunit[(j*6) + 2] = 1;  /* gradient for lower bound */
	o8gri_1.gunit[(j*6) + 3] = 1;
	o8gri_1.gunit[(j*6) + 4] = j; 
	o8gri_1.gunit[(j*6) + 5] = -1; /* gradient for upper bound */
    }

    return 0;
} 

/* /////////////////////////////////////////////////////////////// */

int ef_(x, fx)
double *x, *fx;
{
  /* uses global variables o8dim_1.n, opt_g, and opt_g0 */

  register long i,j;

  (*fx)=0;

  /* compute linear part */
  for(i=0;i<o8dim_1.n;i++) {
    (*fx)+=x[i]*opt_g0[i];
  }

  /* compute quadratic part */
  for(i=0;i<o8dim_1.n;i++) {
      (*fx)+=0.5*x[i]*x[i]*opt_g[o8dim_1.n*i+i];
      for(j=i+1;j<o8dim_1.n;j++) {
	(*fx)+=x[i]*x[j]*opt_g[o8dim_1.n*i+j];
      }
  }

  return(0);
}

/* /////////////////////////////////////////////////////////////// */

int egradf_(x, gradf)
double *x, *gradf;
{
  register long i,j;
  register double temp;

  for(i=0;i<o8dim_1.n;i++) {
    temp=0;
    for(j=0;j<=i;j++) {
      temp+=x[j]*opt_g[o8dim_1.n*j+i];
    }
    for(j=i+1;j<o8dim_1.n;j++) {
      temp+=x[j]*opt_g[o8dim_1.n*i+j];
    }
    gradf[i]=opt_g0[i]+temp;
  }

  return 0;
} 

/* /////////////////////////////////////////////////////////////// */

int eh_(n, x, hx)
long *n;         /* n is always 0 */
double *x, *hx;
{
  /* uses the global variables opt_ce and opt_ce0 */
  register long i;

  (*hx)=opt_ce0[0];
  for(i=0;i<o8dim_1.n;i++) {
    (*hx)+=x[i]*opt_ce[i];
  }

  return 0;
} 

int egradh_(n, x, gradhn)
long *n;        /* n is always 0 */
double *x, *gradhn;
{
  register long i;

  for(i=0;i<o8dim_1.n;i++) {
    gradhn[i]=opt_ce[i];
  }
  
  return 0;
} 

/* /////////////////////////////////////////////////////////////// */

int eg_(n, x, gxn)
long *n;
double *x, *gxn;
{
  /* uses global variable svm_c */

  if(((*n)-1) % 2) {
    (*gxn)=svm_c-x[(int)(((*n)-2)/2)];
  }
  else {
    (*gxn)=x[(int)(((*n)-1)/2)];
  }

  return 0;
} 

int egradg_(n, x, gradgn)  /* this function is not really used */
long *n;                   /* look at gunit in function setup0 */
double *x, *gradgn;
{
  register long j;

  for(j=0;j<o8dim_1.n;j++) {
    gradgn[j]=0;
  }
  if(((*n)-1) % 2) {
    gradgn[(int)(((*n)-2)/2)]=-1;
  }
  else {
    gradgn[(int)(((*n)-1)/2)]=1;
  }
  
  return 0;
} 

/* //////////////////////////////////////////////////////////// */













